@@ -4,7 +4,6 @@
 
 #include "daemon/io.h"
 
-#include <assert.h>
 #include <contrib/ucw/lib.h>
 #include <contrib/ucw/mempool.h>
 #include <libknot/errcode.h>
@@ -12,6 +11,7 @@
 #include <sys/resource.h>
 
 #if ENABLE_XDP
+	#include <libknot/xdp/eth.h>
 	#include <libknot/xdp/xdp.h>
 	#include <net/if.h>
 #endif
@@ -76,16 +76,17 @@ void udp_recv(uv_udp_t *handle, ssize_t nread, const uv_buf_t *buf,
 
 	if (session_flags(s)->outgoing) {
 		const struct sockaddr *peer = session_get_peer(s);
-		assert(peer->sa_family != AF_UNSPEC);
+		if (kr_fails_assert(peer->sa_family != AF_UNSPEC))
+			return;
 		if (kr_sockaddr_cmp(peer, addr) != 0) {
-			kr_log_verbose("[io] <= ignoring UDP from unexpected address '%s'\n",
+			kr_log_debug(IO, "<= ignoring UDP from unexpected address '%s'\n",
 					kr_straddr(addr));
 			return;
 		}
 	}
 	ssize_t consumed = session_wirebuf_consume(s, (const uint8_t *)buf->base,
 						   nread);
-	assert(consumed == nread); (void)consumed;
+	kr_assert(consumed == nread);
 	session_wirebuf_process(s, addr);
 	session_wirebuf_discard(s);
 	mp_flush(the_worker->pkt_pool.ctx);
@@ -164,8 +165,8 @@ int io_bind(const struct sockaddr *addr, int type, const endpoint_flags_t *flags
 		int omit = IP_PMTUDISC_OMIT;
 		if (type == SOCK_DGRAM && addr->sa_family == AF_INET
 		    && setsockopt(fd, IPPROTO_IP, IP_MTU_DISCOVER, &omit, sizeof(omit))) {
-			kr_log_error(
-				"[ io ] failed to disable Path MTU discovery for %s UDP: %s\n",
+			kr_log_error(IO,
+				"failed to disable Path MTU discovery for %s UDP: %s\n",
 				kr_straddr(addr), strerror(errno));
 		}
 #endif
@@ -192,13 +193,13 @@ int io_listen_udp(uv_loop_t *loop, uv_udp_t *handle, int fd)
 	check_bufsize(h);
 	/* Handle is already created, just create context. */
 	struct session *s = session_new(h, false, false);
-	assert(s);
+	kr_require(s);
 	session_flags(s)->outgoing = false;
 
 	int socklen = sizeof(union inaddr);
 	ret = uv_udp_getsockname(handle, session_get_sockname(s), &socklen);
 	if (ret) {
-		kr_log_error("ERROR: getsockname failed: %s\n", uv_strerror(ret));
+		kr_log_error(IO, "ERROR: getsockname failed: %s\n", uv_strerror(ret));
 		abort(); /* It might be nontrivial not to leak something here. */
 	}
 
@@ -209,7 +210,8 @@ void tcp_timeout_trigger(uv_timer_t *timer)
 {
 	struct session *s = timer->data;
 
-	assert(!session_flags(s)->closing);
+	if (kr_fails_assert(!session_flags(s)->closing))
+		return;
 
 	if (!session_tasklist_is_empty(s)) {
 		int finalized = session_tasklist_finalize_expired(s);
@@ -251,7 +253,7 @@ void tcp_timeout_trigger(uv_timer_t *timer)
 		} else {
 			struct sockaddr *peer = session_get_peer(s);
 			char *peer_str = kr_straddr(peer);
-			kr_log_verbose("[io] => closing connection to '%s'\n",
+			kr_log_debug(IO, "=> closing connection to '%s'\n",
 				       peer_str ? peer_str : "");
 			if (session_flags(s)->outgoing) {
 				worker_del_tcp_waiting(the_worker, peer);
@@ -265,8 +267,8 @@ void tcp_timeout_trigger(uv_timer_t *timer)
 static void tcp_recv(uv_stream_t *handle, ssize_t nread, const uv_buf_t *buf)
 {
 	struct session *s = handle->data;
-	assert(s && session_get_handle(s) == (uv_handle_t *)handle &&
-	       handle->type == UV_TCP);
+	if (kr_fails_assert(s && session_get_handle(s) == (uv_handle_t *)handle && handle->type == UV_TCP))
+		return;
 
 	if (session_flags(s)->closing) {
 		return;
@@ -279,10 +281,10 @@ static void tcp_recv(uv_stream_t *handle, ssize_t nread, const uv_buf_t *buf)
 	}
 
 	if (nread < 0 || !buf->base) {
-		if (kr_verbose_status) {
+		if (kr_log_is_debug(IO, NULL)) {
 			struct sockaddr *peer = session_get_peer(s);
 			char *peer_str = kr_straddr(peer);
-			kr_log_verbose("[io] => connection to '%s' closed by peer (%s)\n",
+			kr_log_debug(IO, "=> connection to '%s' closed by peer (%s)\n",
 				       peer_str ? peer_str : "",
 				       uv_strerror(nread));
 		}
@@ -298,10 +300,10 @@ static void tcp_recv(uv_stream_t *handle, ssize_t nread, const uv_buf_t *buf)
 		   Decode data free space in session wire buffer. */
 		consumed = tls_process_input_data(s, (const uint8_t *)buf->base, nread);
 		if (consumed < 0) {
-			if (kr_verbose_status) {
+			if (kr_log_is_debug(IO, NULL)) {
 				struct sockaddr *peer = session_get_peer(s);
 				char *peer_str = kr_straddr(peer);
-				kr_log_verbose("[io] => connection to '%s': "
+				kr_log_debug(IO, "=> connection to '%s': "
 					       "error processing TLS data, close\n",
 					       peer_str ? peer_str : "");
 			}
@@ -317,10 +319,10 @@ static void tcp_recv(uv_stream_t *handle, ssize_t nread, const uv_buf_t *buf)
 	if (session_flags(s)->has_http) {
 		consumed = http_process_input_data(s, data, data_len);
 		if (consumed < 0) {
-			if (kr_verbose_status) {
+			if (kr_log_is_debug(IO, NULL)) {
 				struct sockaddr *peer = session_get_peer(s);
 				char *peer_str = kr_straddr(peer);
-				kr_log_verbose("[io] => connection to '%s': "
+				kr_log_debug(IO, "=> connection to '%s': "
 				       "error processing HTTP data, close\n",
 				       peer_str ? peer_str : "");
 			}
@@ -337,7 +339,7 @@ static void tcp_recv(uv_stream_t *handle, ssize_t nread, const uv_buf_t *buf)
 	/* data points to start of the free space in session wire buffer.
 	   Simple increase internal counter. */
 	consumed = session_wirebuf_consume(s, data, data_len);
-	assert(consumed == data_len);
+	kr_assert(consumed == data_len);
 
 	int ret = session_wirebuf_process(s, session_get_peer(s));
 	if (ret < 0) {
@@ -353,11 +355,11 @@ static ssize_t tls_send(const uint8_t *buf, const size_t len, struct session *se
 {
 	struct tls_ctx *ctx = session_tls_get_server_ctx(session);
 	ssize_t sent = 0;
-	assert(ctx);
+	kr_require(ctx);
 
 	sent = gnutls_record_send(ctx->c.tls_session, buf, len);
 	if (sent < 0) {
-		kr_log_verbose("[http] gnutls_record_send failed: %s (%zd)\n",
+		kr_log_debug(DOH, "gnutls_record_send failed: %s (%zd)\n",
 			       gnutls_strerror_name(sent), sent);
 		return kr_error(EIO);
 	}
@@ -392,8 +394,8 @@ static void _tcp_accept(uv_stream_t *master, int status, bool tls, bool http)
 
 	/* struct session was allocated \ borrowed from memory pool. */
 	struct session *s = client->data;
-	assert(session_flags(s)->outgoing == false);
-	assert(session_flags(s)->has_tls == tls);
+	kr_require(session_flags(s)->outgoing == false);
+	kr_require(session_flags(s)->has_tls == tls);
 
 	if (uv_accept(master, (uv_stream_t *)client) != 0) {
 		/* close session, close underlying uv handles and
@@ -515,7 +517,7 @@ int io_listen_tcp(uv_loop_t *loop, uv_tcp_t *handle, int fd, int tcp_backlog, bo
 #if ENABLE_DOH2
 		connection = https_accept;
 #else
-		kr_log_error("[ io ] kresd was compiled without libnghttp2 support\n");
+		kr_log_error(IO, "kresd was compiled without libnghttp2 support\n");
 		return kr_error(ENOPROTOOPT);
 #endif
 	} else if (has_tls) {
@@ -534,7 +536,7 @@ int io_listen_tcp(uv_loop_t *loop, uv_tcp_t *handle, int fd, int tcp_backlog, bo
 #ifdef TCP_DEFER_ACCEPT
 	val = KR_CONN_RTT_MAX/1000;
 	if (setsockopt(fd, IPPROTO_TCP, TCP_DEFER_ACCEPT, &val, sizeof(val))) {
-		kr_log_error("[ io ] listen TCP (defer_accept): %s\n", strerror(errno));
+		kr_log_error(IO, "listen TCP (defer_accept): %s\n", strerror(errno));
 	}
 #endif
 
@@ -551,7 +553,7 @@ int io_listen_tcp(uv_loop_t *loop, uv_tcp_t *handle, int fd, int tcp_backlog, bo
 	val = 1; /* Accepts on/off */
 	#endif
 	if (setsockopt(fd, IPPROTO_TCP, TCP_FASTOPEN, &val, sizeof(val))) {
-		kr_log_error("[ io ] listen TCP (fastopen): %s%s\n", strerror(errno),
+		kr_log_error(IO, "listen TCP (fastopen): %s%s\n", strerror(errno),
 			(errno != EPERM ? "" :
 			 ".  This may be caused by TCP Fast Open being disabled in the OS."));
 	}
@@ -673,18 +675,18 @@ void io_tty_process_input(uv_stream_t *stream, ssize_t nread, const uv_buf_t *bu
 			goto next_iter;
 		}
 
-		int ret = engine_cmd(L, cmd, false);
+		const bool cmd_failed = engine_cmd(L, cmd, false);
 		const char *message = NULL;
 		size_t len_s;
 		if (lua_gettop(L) > 0) {
 			message = lua_tolstring(L, -1, &len_s);
 		}
 
-		/* Simpler output in binary mode */
+		/* Send back the output, either in "binary" or normal mode. */
 		if (data->mode == io_mode_binary) {
 			/* Leader expects length field in all cases */
 			if (!message || len_s > UINT32_MAX) {
-				kr_log_error("[io] unrepresentable respose on control socket, "
+				kr_log_error(IO, "unrepresentable respose on control socket, "
 						"sending back empty block (command '%s')\n", cmd);
 				len_s = 0;
 			}
@@ -692,28 +694,24 @@ void io_tty_process_input(uv_stream_t *stream, ssize_t nread, const uv_buf_t *bu
 			fwrite(&len_n, sizeof(len_n), 1, out);
 			if (len_s > 0)
 				fwrite(message, len_s, 1, out);
-			goto next_iter;
-		}
-
-		/* Log to remote socket if connected */
-		const char *delim = args->quiet ? "" : "> ";
-		if (stream_fd != STDIN_FILENO) {
-			if (VERBOSE_STATUS)
-				fprintf(stdout, "%s\n", cmd); /* Duplicate command to logs */
+		} else {
 			if (message)
-				fprintf(out, "%s", message); /* Duplicate output to sender */
+				fprintf(out, "%s", message);
 			if (message || !args->quiet)
 				fprintf(out, "\n");
-			fprintf(out, "%s", delim);
+			if (!args->quiet)
+				fprintf(out, "> ");
 		}
-		if (stream_fd == STDIN_FILENO || VERBOSE_STATUS) {
-			/* Log to standard streams */
-			FILE *fp_out = ret ? stderr : stdout;
+
+		/* Duplicate command and output to logs */
+		if (cmd_failed) {
+			kr_log_warning(CONTROL, "> %s\n", cmd);
 			if (message)
-				fprintf(fp_out, "%s", message);
-			if (message && !args->quiet)
-				fprintf(fp_out, "\n");
-			fprintf(fp_out, "%s", delim);
+				kr_log_warning(CONTROL, "%s\n", message);
+		} else {
+			kr_log_debug(CONTROL, "> %s\n", cmd);
+			if (message)
+				kr_log_debug(CONTROL, "%s\n", message);
 		}
 	next_iter:
 		lua_settop(L, 0); /* not required in some cases but harmless */
@@ -796,16 +794,16 @@ static void xdp_rx(uv_poll_t* handle, int status, int events)
 {
 	const int XDP_RX_BATCH_SIZE = 64;
 	if (status < 0) {
-		kr_log_error("[xdp] poll status %d: %s\n", status, uv_strerror(status));
+		kr_log_error(XDP, "poll status %d: %s\n", status, uv_strerror(status));
 		return;
 	}
 	if (events != UV_READABLE) {
-		kr_log_error("[xdp] poll unexpected events: %d\n", events);
+		kr_log_error(XDP, "poll unexpected events: %d\n", events);
 		return;
 	}
 
 	xdp_handle_data_t *xhd = handle->data;
-	assert(xhd && xhd->session && xhd->socket);
+	kr_require(xhd && xhd->session && xhd->socket);
 	uint32_t rcvd;
 	knot_xdp_msg_t msgs[XDP_RX_BATCH_SIZE];
 	int ret = knot_xdp_recv(xhd->socket, msgs, XDP_RX_BATCH_SIZE, &rcvd
@@ -813,18 +811,17 @@ static void xdp_rx(uv_poll_t* handle, int status, int events)
 			, NULL
 			#endif
 			);
-	if (ret == KNOT_EOK) {
-		kr_log_verbose("[xdp] poll triggered, processing a batch of %d packets\n",
-			(int)rcvd);
-	} else {
-		kr_log_error("[xdp] knot_xdp_recv(): %d, %s\n", ret, knot_strerror(ret));
-		assert(false); // ATM it can only be returned when called incorrectly
+
+	if (kr_fails_assert(ret == KNOT_EOK)) {
+		/* ATM other error codes can only be returned when called incorrectly */
+		kr_log_error(XDP, "knot_xdp_recv(): %d, %s\n", ret, knot_strerror(ret));
 		return;
 	}
-	assert(rcvd <= XDP_RX_BATCH_SIZE);
+	kr_log_debug(XDP, "poll triggered, processing a batch of %d packets\n", (int)rcvd);
+	kr_require(rcvd <= XDP_RX_BATCH_SIZE);
 	for (int i = 0; i < rcvd; ++i) {
 		const knot_xdp_msg_t *msg = &msgs[i];
-		assert(msg->payload.iov_len <= KNOT_WIRE_MAX_PKTSIZE);
+		kr_require(msg->payload.iov_len <= KNOT_WIRE_MAX_PKTSIZE);
 		knot_pkt_t *kpkt = knot_pkt_new(msg->payload.iov_base, msg->payload.iov_len,
 						&the_worker->pkt_pool);
 		if (kpkt == NULL) {
@@ -836,7 +833,7 @@ static void xdp_rx(uv_poll_t* handle, int status, int events)
 					msg->eth_from, msg->eth_to, kpkt);
 		}
 		if (ret)
-			kr_log_verbose("[xdp] worker_submit() == %d: %s\n", ret, kr_strerror(ret));
+			kr_log_debug(XDP, "worker_submit() == %d: %s\n", ret, kr_strerror(ret));
 		mp_flush(the_worker->pkt_pool.ctx);
 	}
 	knot_xdp_recv_finish(xhd->socket, msgs, rcvd);
@@ -844,10 +841,12 @@ static void xdp_rx(uv_poll_t* handle, int status, int events)
 /// Warn if the XDP program is running in emulated mode (XDP_SKB)
 static void xdp_warn_mode(const char *ifname)
 {
-	assert(ifname);
+	if (kr_fails_assert(ifname))
+		return;
+
 	const unsigned if_index = if_nametoindex(ifname);
 	if (!if_index) {
-		kr_log_info("[xdp] warning: interface %s, unexpected error when converting its name: %s\n",
+		kr_log_warning(XDP, "warning: interface %s, unexpected error when converting its name: %s\n",
 				ifname, strerror(errno));
 		return;
 	}
@@ -857,13 +856,13 @@ static void xdp_warn_mode(const char *ifname)
 	case KNOT_XDP_MODE_FULL:
 		return;
 	case KNOT_XDP_MODE_EMUL:
-		kr_log_info("[xdp] warning: interface %s running only with XDP emulation\n",
+		kr_log_warning(XDP, "warning: interface %s running only with XDP emulation\n",
 				ifname);
 		return;
 	case KNOT_XDP_MODE_NONE: // enum warnings from compiler
 		break;
 	}
-	kr_log_info("[xdp] warning: interface %s running in unexpected XDP mode %d\n",
+	kr_log_warning(XDP, "warning: interface %s running in unexpected XDP mode %d\n",
 			ifname, (int)mode);
 }
 int io_listen_xdp(uv_loop_t *loop, struct endpoint *ep, const char *ifname)
@@ -896,11 +895,10 @@ int io_listen_xdp(uv_loop_t *loop, struct endpoint *ep, const char *ifname)
 	if (!ret) xdp_warn_mode(ifname);
 
 	if (!ret) ret = uv_idle_init(loop, &xhd->tx_waker);
-	if (ret) {
+	if (ret || kr_fails_assert(xhd->socket)) {
 		free(xhd);
-		return kr_error(ret);
+		return ret == 0 ? kr_error(EINVAL) : kr_error(ret);
 	}
-	assert(xhd->socket);
 	xhd->tx_waker.data = xhd->socket;
 
 	ep->fd = knot_xdp_socket_fd(xhd->socket); // probably not useful
@@ -913,7 +911,7 @@ int io_listen_xdp(uv_loop_t *loop, struct endpoint *ep, const char *ifname)
 
 	// beware: this sets poll_handle->data
 	xhd->session = session_new(ep->handle, false, false);
-	assert(!session_flags(xhd->session)->outgoing);
+	kr_require(!session_flags(xhd->session)->outgoing);
 	session_get_sockname(xhd->session)->sa_family = AF_XDP; // to have something in there
 
 	ep->handle->data = xhd;
@@ -958,7 +956,7 @@ static void io_deinit(uv_handle_t *handle)
 		knot_xdp_deinit(xhd->socket);
 		free(xhd);
 	#else
-		assert(false);
+		kr_assert(false);
 	#endif
 	}
 }
@@ -977,7 +975,7 @@ int io_start_read(uv_handle_t *handle)
 	case UV_TCP:
 		return uv_read_start((uv_stream_t *)handle, &handle_getbuf, &tcp_recv);
 	default:
-		assert(!EINVAL);
+		kr_assert(false);
 		return kr_error(EINVAL);
 	}
 }
