@@ -621,7 +621,7 @@ int qr_task_on_send(struct qr_task *task, const uv_handle_t *handle, int status)
 			return status;
 		// start the timer
 		struct kr_query *qry = array_tail(task->ctx->req.rplan.pending);
-		if (kr_fails_assert(qry))
+		if (kr_fails_assert(qry && task->transport))
 			return status;
 		size_t timeout = task->transport->timeout;
 		int ret = session_timer_start(s, on_udp_timeout, timeout, 0);
@@ -1738,11 +1738,13 @@ int worker_submit(struct session *session,
 	    (is_query == is_outgoing)) {
 		if (!is_outgoing) {
 			the_worker->stats.dropped += 1;
+		#if ENABLE_DOH2
 			if (http_ctx) {
 				struct http_stream stream = queue_head(http_ctx->streams);
 				http_free_headers(stream.headers);
 				queue_pop(http_ctx->streams);
 			}
+		#endif
 		}
 		return kr_error(EILSEQ);
 	}
