@@ -2,7 +2,6 @@
  *  SPDX-License-Identifier: GPL-3.0-or-later
 */
 
-#include <assert.h>
 #include <fcntl.h>
 #include <stdbool.h>
 #include <stdlib.h>
@@ -88,7 +87,7 @@ static int lmdb_error(int error)
 	case MDB_TXN_FULL:
 		return kr_error(ENOSPC);
 	default:
-		kr_log_error("[cache] LMDB error: %s\n", mdb_strerror(error));
+		kr_log_error(CACHE, "LMDB error: %s\n", mdb_strerror(error));
 		return kr_error(error);
 	}
 }
@@ -117,7 +116,7 @@ static int refresh_mapsize(struct lmdb_env *env)
 
 	env->mapsize = info.me_mapsize;
 	if (env->mapsize != env->st_size) {
-		kr_log_info("[cache] suspicious size of cache file '%s'"
+		kr_log_info(CACHE, "suspicious size of cache file '%s'"
 				": file size %zu != LMDB map size %zu\n",
 				env->mdb_data_path, (size_t)env->st_size, env->mapsize);
 	}
@@ -129,10 +128,10 @@ static void clear_stale_readers(struct lmdb_env *env)
 	int cleared;
 	int ret = mdb_reader_check(env->env, &cleared);
 	if (ret != MDB_SUCCESS) {
-		kr_log_error("[cache] failed to clear stale reader locks: "
+		kr_log_error(CACHE, "failed to clear stale reader locks: "
 				"LMDB error %d %s\n", ret, mdb_strerror(ret));
 	} else if (cleared != 0) {
-		kr_log_info("[cache] cleared %d stale reader locks\n", cleared);
+		kr_log_info(CACHE, "cleared %d stale reader locks\n", cleared);
 	}
 }
 
@@ -145,14 +144,14 @@ static void clear_stale_readers(struct lmdb_env *env)
  */
 static int txn_get_noresize(struct lmdb_env *env, unsigned int flag, MDB_txn **txn)
 {
-	assert(!env->txn.rw && (!env->txn.ro || !env->txn.ro_active));
+	if (kr_fails_assert(!env->txn.rw && (!env->txn.ro || !env->txn.ro_active)))
+		return kr_error(1);
 	int attempts = 0;
 	int ret;
 retry:
 	/* Do a few attempts in case we encounter multiple issues at once. */
-	if (++attempts > 2) {
+	if (++attempts > 2)
 		return kr_error(1);
-	}
 
 	if (flag == FLAG_RENEW) {
 		ret = mdb_txn_renew(*txn);
@@ -161,7 +160,7 @@ retry:
 	}
 
 	if (unlikely(ret == MDB_MAP_RESIZED)) {
-		kr_log_info("[cache] detected size increased by another process\n");
+		kr_log_info(CACHE, "detected size increased by another process\n");
 		ret = refresh_mapsize(env);
 		if (ret == 0)
 			goto retry;
@@ -175,7 +174,8 @@ retry:
 /** Obtain a transaction.  (they're cached in env->txn) */
 static int txn_get(struct lmdb_env *env, MDB_txn **txn, bool rdonly)
 {
-	assert(env && txn);
+	if (kr_fails_assert(env && txn))
+		return kr_error(EINVAL);
 	if (env->txn.rw) {
 		/* Reuse the *open* RW txn even if only reading is requested.
 		 * We leave the management of this to the cdb_commit command.
@@ -194,7 +194,7 @@ static int txn_get(struct lmdb_env *env, MDB_txn **txn, bool rdonly)
 		int ret = txn_get_noresize(env, 0/*RW*/, &env->txn.rw);
 		if (ret == MDB_SUCCESS) {
 			*txn = env->txn.rw;
-			assert(*txn);
+			kr_assert(*txn);
 		}
 		return lmdb_error(ret);
 	}
@@ -211,7 +211,7 @@ static int txn_get(struct lmdb_env *env, MDB_txn **txn, bool rdonly)
 	}
 	env->txn.ro_active = true;
 	*txn = env->txn.ro;
-	assert(*txn);
+	kr_assert(*txn);
 	return kr_ok();
 }
 
@@ -234,10 +234,10 @@ static int cdb_commit(kr_cdb_pt db, struct kr_cdb_stats *stats)
 /** Obtain a read-only cursor (and a read-only transaction). */
 static int txn_curs_get(struct lmdb_env *env, MDB_cursor **curs, struct kr_cdb_stats *stats)
 {
-	assert(env && curs);
-	if (env->txn.ro_curs_active) {
+	if (kr_fails_assert(env && curs))
+		return kr_error(EINVAL);
+	if (env->txn.ro_curs_active)
 		goto success;
-	}
 	/* Only in a read-only txn; TODO: it's a bit messy/coupled */
 	if (env->txn.rw) {
 		int ret = cdb_commit(env2db(env), stats);
@@ -255,10 +255,10 @@ static int txn_curs_get(struct lmdb_env *env, MDB_cursor **curs, struct kr_cdb_s
 	if (ret) return lmdb_error(ret);
 	env->txn.ro_curs_active = true;
 success:
-	assert(env->txn.ro_curs_active && env->txn.ro && env->txn.ro_active
-		&& !env->txn.rw);
+	kr_assert(env->txn.ro_curs_active && env->txn.ro && env->txn.ro_active
+			 && !env->txn.rw);
 	*curs = env->txn.ro_curs;
-	assert(*curs);
+	kr_assert(*curs);
 	return kr_ok();
 }
 
@@ -291,7 +291,8 @@ static void txn_abort(struct lmdb_env *env)
 /*! \brief Close the database. */
 static void cdb_close_env(struct lmdb_env *env, struct kr_cdb_stats *stats)
 {
-	assert(env && env->env);
+	if (kr_fails_assert(env && env->env))
+		return;
 
 	/* Get rid of any transactions. */
 	txn_free_ro(env);
@@ -383,7 +384,7 @@ static int cdb_open_env(struct lmdb_env *env, const char *path, const size_t map
 	if (ret == EINVAL) {
 		/* POSIX says this can happen when the feature isn't supported by the FS.
 		 * We haven't seen this happen on Linux+glibc but it was reported on FreeBSD.*/
-		kr_log_info("[cache] space pre-allocation failed and ignored; "
+		kr_log_info(CACHE, "space pre-allocation failed and ignored; "
 				"your (file)system probably doesn't support it.\n");
 	} else if (ret != 0) {
 		mdb_txn_abort(txn);
@@ -485,7 +486,7 @@ static int cdb_check_health(kr_cdb_pt db, struct kr_cdb_stats *stats)
 	}
 
 	if (st.st_dev != env->st_dev || st.st_ino != env->st_ino) {
-		kr_log_verbose("[cache] cache file has been replaced, reopening\n");
+		kr_log_debug(CACHE, "cache file has been replaced, reopening\n");
 		int ret = reopen_env(env, stats, 0); // we accept mapsize from the new file
 		return ret == 0 ? 1 : ret;
 	}
@@ -494,7 +495,7 @@ static int cdb_check_health(kr_cdb_pt db, struct kr_cdb_stats *stats)
 	 * contrary to methods based on mdb_env_info(). */
 	if (st.st_size == env->st_size)
 		return kr_ok();
-	kr_log_info("[cache] detected size change (by another instance?) of file '%s'"
+	kr_log_info(CACHE, "detected size change (by another instance?) of file '%s'"
 			": file size %zu -> file size %zu\n",
 			env->mdb_data_path, (size_t)env->st_size, (size_t)st.st_size);
 	env->st_size = st.st_size; // avoid retrying in cycle even if we fail
@@ -505,7 +506,8 @@ static int cdb_check_health(kr_cdb_pt db, struct kr_cdb_stats *stats)
  * The lock is auto-released by OS in case the process finishes in any way (file remains). */
 static int lockfile_get(const char *path)
 {
-	assert(path);
+	if (kr_fails_assert(path))
+		return kr_error(EINVAL);
 	const int fd = open(path, O_CREAT|O_RDWR, S_IRUSR|S_IWUSR|S_IRGRP|S_IWGRP);
 	if (fd < 0)
 		return kr_error(errno);
@@ -530,7 +532,8 @@ static int lockfile_get(const char *path)
 /** Release and remove lockfile created by lockfile_get().  Return kr_error(). */
 static int lockfile_release(int fd)
 {
-	assert(fd > 0); // fd == 0 is surely a mistake, in our case at least
+	if (kr_fails_assert(fd > 0)) // fd == 0 is surely a mistake, in our case at least
+		return kr_error(EINVAL);
 	if (close(fd)) {
 		return kr_error(errno);
 	} else {
@@ -555,7 +558,7 @@ static int cdb_clear(kr_cdb_pt db, struct kr_cdb_stats *stats)
 				return ret;
 			}
 		}
-		kr_log_info("[cache] clearing error, falling back\n");
+		kr_log_info(CACHE, "clearing error, falling back\n");
 	}
 	/* Fallback: we'll remove the database files and reopen.
 	 * Other instances can continue to use the removed lmdb,
@@ -579,7 +582,7 @@ static int cdb_clear(kr_cdb_pt db, struct kr_cdb_stats *stats)
 	/* Find if we get a lock on lockfile. */
 	const int lockfile_fd = lockfile_get(lockfile);
 	if (lockfile_fd < 0) {
-		kr_log_error("[cache] clearing failed to get ./krcachelock (%s); retry later\n",
+		kr_log_error(CACHE, "clearing failed to get ./krcachelock (%s); retry later\n",
 				kr_strerror(lockfile_fd));
 		/* As we're out of space (almost certainly - mdb_drop didn't work),
 		 * we will retry on the next failing write operation. */
@@ -595,7 +598,7 @@ static int cdb_clear(kr_cdb_pt db, struct kr_cdb_stats *stats)
 			ret = kr_ok();
 		// else pass some other error
 	} else {
-		kr_log_verbose("[cache] clear: identical files, unlinking\n");
+		kr_log_debug(CACHE, "clear: identical files, unlinking\n");
 		// coverity[toctou]
 		unlink(env->mdb_data_path);
 		unlink(mdb_lockfile);
@@ -605,7 +608,7 @@ static int cdb_clear(kr_cdb_pt db, struct kr_cdb_stats *stats)
 	/* Environment updated, release lockfile. */
 	int lrerr = lockfile_release(lockfile_fd);
 	if (lrerr) {
-		kr_log_error("[cache] failed to release ./krcachelock: %s\n",
+		kr_log_error(CACHE, "failed to release ./krcachelock: %s\n",
 				kr_strerror(lrerr));
 	}
 	return ret;
@@ -780,7 +783,8 @@ static int cdb_match(kr_cdb_pt db, struct kr_cdb_stats *stats,
 static int cdb_read_leq(kr_cdb_pt db, struct kr_cdb_stats *stats,
 		knot_db_val_t *key, knot_db_val_t *val)
 {
-	assert(db && key && key->data && val);
+	if (kr_fails_assert(db && key && key->data && val))
+		return kr_error(EINVAL);
 	struct lmdb_env *env = db2env(db);
 	MDB_cursor *curs = NULL;
 	int ret = txn_curs_get(env, &curs, stats);
