@@ -16,7 +16,6 @@
  */
 
 #include <sys/time.h>
-#include <assert.h>
 #include <arpa/inet.h>
 
 #include <contrib/cleanup.h>
@@ -33,8 +32,9 @@
 #include "lib/module.h"
 #include "lib/dnssec/ta.h"
 
-#define VERBOSE_MSG(...) QRVERBOSE(req->current_query, "iter", __VA_ARGS__)
-#define QVERBOSE_MSG(qry, ...) QRVERBOSE(qry, "iter", __VA_ARGS__)
+#define VERBOSE_MSG(...) QRVERBOSE(req->current_query, ITERATOR, __VA_ARGS__)
+#define QVERBOSE_MSG(qry, ...) QRVERBOSE(qry, ITERATOR, __VA_ARGS__)
+#define WITH_VERBOSE(qry) if (kr_log_is_debug_qry(ITERATOR, (qry)))
 
 /* Iterator often walks through packet section, this is an abstraction. */
 typedef int (*rr_callback_t)(const knot_rrset_t *, unsigned, struct kr_request *);
@@ -308,7 +308,7 @@ static int update_cut(knot_pkt_t *pkt, const knot_rrset_t *rr,
 			continue;
 		}
 		int ret = kr_zonecut_add(cut, ns_name, NULL, 0);
-		assert(!ret); (void)ret;
+		kr_assert(!ret);
 
 		/* Choose when to use glue records. */
 		const bool in_bailiwick =
@@ -390,7 +390,8 @@ static int pick_authority(knot_pkt_t *pkt, struct kr_request *req, bool to_wire)
 static int process_authority(knot_pkt_t *pkt, struct kr_request *req)
 {
 	struct kr_query *qry = req->current_query;
-	assert(!(qry->flags.STUB));
+	if (kr_fails_assert(!qry->flags.STUB))
+		return KR_STATE_FAIL;
 
 	int result = KR_STATE_CONSUME;
 	if (qry->flags.FORWARD) {
@@ -493,7 +494,8 @@ static int finalize_answer(knot_pkt_t *pkt, struct kr_request *req)
 static int unroll_cname(knot_pkt_t *pkt, struct kr_request *req, bool referral, const knot_dname_t **cname_ret)
 {
 	struct kr_query *query = req->current_query;
-	assert(!(query->flags.STUB));
+	if (kr_fails_assert(!query->flags.STUB))
+		return KR_STATE_FAIL;
 	/* Process answer type */
 	const knot_pktsection_t *an = knot_pkt_section(pkt, KNOT_ANSWER);
 	const knot_dname_t *cname = NULL;
@@ -791,7 +793,7 @@ static int process_answer(knot_pkt_t *pkt, struct kr_request *req)
 		next->cname_parent = query;
 		/* Want DNSSEC if and only if it's posible to secure
 		 * this name (i.e. iff it is covered by a TA) */
-		if (kr_ta_covers_qry(req->ctx, cname, query->stype)) {
+		if (kr_ta_closest(req->ctx, cname, query->stype)) {
 			next->flags.DNSSEC_WANT = true;
 		} else {
 			next->flags.DNSSEC_WANT = false;
@@ -848,7 +850,8 @@ static int process_answer(knot_pkt_t *pkt, struct kr_request *req)
 static int process_stub(knot_pkt_t *pkt, struct kr_request *req)
 {
 	struct kr_query *query = req->current_query;
-	assert(query->flags.STUB);
+	if (kr_fails_assert(query->flags.STUB))
+		return KR_STATE_FAIL;
 	/* Pick all answer RRs. */
 	const knot_pktsection_t *an = knot_pkt_section(pkt, KNOT_ANSWER);
 	for (unsigned i = 0; i < an->count; ++i) {
@@ -942,7 +945,8 @@ int kr_make_query(struct kr_query *query, knot_pkt_t *pkt)
 
 static int prepare_query(kr_layer_t *ctx, knot_pkt_t *pkt)
 {
-	assert(pkt && ctx);
+	if (kr_fails_assert(pkt && ctx))
+		return KR_STATE_FAIL;
 	struct kr_request *req = ctx->req;
 	struct kr_query *query = req->current_query;
 	if (!query || ctx->state & (KR_STATE_DONE|KR_STATE_FAIL)) {
@@ -994,7 +998,8 @@ static bool satisfied_by_additional(const struct kr_query *qry)
  */
 static int resolve(kr_layer_t *ctx, knot_pkt_t *pkt)
 {
-	assert(pkt && ctx);
+	if (kr_fails_assert(pkt && ctx))
+		return KR_STATE_FAIL;
 	struct kr_request *req = ctx->req;
 	struct kr_query *query = req->current_query;
 	if (!query) {
@@ -1060,9 +1065,8 @@ static int resolve(kr_layer_t *ctx, knot_pkt_t *pkt)
 	 * The most important part is to check for spoofing: is_paired_to_query() */
 	query->flags.PKT_IS_SANE = true;
 
-#ifndef NOVERBOSELOG
-	const knot_lookup_t *rcode = knot_lookup_by_id(knot_rcode_names, knot_wire_get_rcode(pkt->wire));
-#endif
+	const knot_lookup_t *rcode = // just for logging but cheaper than a condition
+		knot_lookup_by_id(knot_rcode_names, knot_wire_get_rcode(pkt->wire));
 
 	// We can't return directly from the switch because we have to give feedback to server selection first
 	int ret = 0;
@@ -1145,7 +1149,7 @@ static int resolve(kr_layer_t *ctx, knot_pkt_t *pkt)
 			 * we trigger another cache *reading* attempt
 			 * for the subsequent PRODUCE round.
 			 */
-			assert(query->flags.NONAUTH);
+			kr_assert(query->flags.NONAUTH);
 			query->flags.CACHE_TRIED = false;
 			VERBOSE_MSG("<= referral response, but cache should stop us short now\n");
 		} else {
