@@ -441,6 +441,8 @@ int kr_straddr_subnet(void *dst, const char *addr)
 	/* Parse subnet */
 	int bit_len = 0;
 	int family = kr_straddr_family(addr);
+	if (family != AF_INET && family != AF_INET6)
+		return kr_error(EINVAL);
 	auto_free char *addr_str = strdup(addr);
 	char *subnet = strchr(addr_str, '/');
 	if (subnet) {
@@ -1010,6 +1012,10 @@ char *kr_pkt_text(const knot_pkt_t *pkt)
 	return result;
 }
 
+const knot_dump_style_t KR_DUMP_STYLE_DEFAULT = { /* almost all = false, */
+	.show_ttl = true,
+};
+
 char *kr_rrset_text(const knot_rrset_t *rr)
 {
 	if (!rr) {
@@ -1019,7 +1025,7 @@ char *kr_rrset_text(const knot_rrset_t *rr)
 	/* Note: knot_rrset_txt_dump will double the size until the rrset fits */
 	size_t bufsize = 128;
 	char *buf = malloc(bufsize);
-	int ret = knot_rrset_txt_dump(rr, &buf, &bufsize, &KNOT_DUMP_STYLE_DEFAULT);
+	int ret = knot_rrset_txt_dump(rr, &buf, &bufsize, &KR_DUMP_STYLE_DEFAULT);
 	if (ret < 0) {
 		free(buf);
 		return NULL;
